///File : DrawableObject.hpp
//------------------------------------------------------------------------------
/*
    This file is part of DevisMaker.

    DevisMaker is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DevisMaker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DevisMaker.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#ifndef MDO_DRAWABLE_OBJECTS_HPP_DEVIS_MAKER
#define MDO_DRAWABLE_OBJECTS_HPP_DEVIS_MAKER
//------------------------------------------------------------------------------
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif
//------------------------------------------------------------------------------
#include "config.hpp"
#include "buildVersionOptions.hpp"
//------------------------------------------------------------------------------
#include <DevisManager.hpp>
//------------------------------------------------------------------------------
#include <wx/treebase.h>
#include "DialogProperty.hpp"
//------------------------------------------------------------------------------
#include "strWxStdConv.hpp"
//------------------------------------------------------------------------------

#define PAGE_WIDTH 2480
#define PAGE_HEIGHT 3508

namespace mdo
{
    class DrawableObject;
    class Sizer;

    enum{vAlign_Left = 0, vAlign_Center = 1, vAlign_Right = 2};
    enum{hAlign_Top = 0, hAlign_Center = 1, hAlign_Bottom = 2};

    const int vAlign_nb = 3;
    const wxString vAlign_Names[vAlign_nb] = {_("Left"), _("Center"), _("Right")};

    const int hAlign_nb = 3;
    const wxString hAlign_Names[hAlign_nb] = {_("Top"), _("Center"), _("Bottom")};

	class DrawableObject
	{
	public:
        // Init / Save
		DrawableObject(ConfigManager *cfg, ConfigManager *devisCfg, DrawableObject *parrentSizer, const std::string &name = "");

		// Cfg / Edit
		virtual void LoadFromCfg(const std::string &name = "") = 0;
		virtual void SaveToCfg(const std::string &name = "") = 0;
		virtual void CreateToCfg(const std::string &name = "") = 0;
		virtual void RemoveFromCfg(const std::string &name = "", const bool &withChilds = true) = 0;
        virtual void EditPropertyDialog(DialogProperty *window) = 0;

        // Autre
        bool IsChildOfSizer(const DrawableObject *sizer) const;

		void AutoChangeConfigName();
		std::string GetConfigName() const;

		std::string GetType() const;

		DrawableObject* GetParent();
		void SetParent(DrawableObject* parentSizer);

		// HumanName
		std::string GetHumanName() const;
		void SetHumanName(const std::string &name);

        // Draw
		virtual void Draw(wxDC *dc, const wxPoint &pos) = 0;

        // Size
        double GetWidthProportion() const;
        //void SetWidthProportion(const double &value);
        double GetHeightProportion() const;
        //void SetHeightProportion(const double &value);

        // La width est prioritaire : la height peut être modifié par le MDO.
		virtual bool CalculSizeReal(wxDC *dc, const int &width, const int &askedHeight = 0) = 0;
		virtual wxSize GetSizeReal() const;

        // Flags
		bool IsOk() const;
        bool IsReadyToDraw() const;

        // Special treeCtrl
        wxTreeItemId GetTreeItemId() const;
        void SetTreeItemId(const wxTreeItemId &treeItemId);

	protected:
        // Throw
        void ThrowIfNotOk() const;
        void ThrowIfNotReadyToDraw() const;

        // Info
		void SetConfigName(const std::string &name);

        // Cfg
		ConfigManager *m_cfg;
		ConfigManager *m_devisCfg;
		std::string m_cfgName;
		std::string m_cfgPrefix;
        std::string m_typeName;

		// Size
		double m_widthProportion, m_heightProportion;
		bool m_isWidthGiven, m_isHeightGiven;
		wxSize m_sizeReal;

		// Flags
		bool m_isReadyToDraw;

		// Special treeCtrl
		wxTreeItemId m_treeItemId;

		// Parent (sizer)
		DrawableObject *m_parentSizer;

		// human name
		std::string m_humanName;
	};

	//------------------------------------------------------------------------------
	wxSize GetTextSize(const wxDC *dc, wxString str);
    wxString ArrangeTextForWidth(const wxDC *dc, wxString str,  const int &width);
    void DrawText(wxDC *dc, wxPoint pos, wxString str, const int &vAlign = vAlign_Left, const int &hAlign = hAlign_Top );
    bool DrawableObjectExistsInCfg(const ConfigManager *cfg, const std::string &type, const std::string &name);

    void DCSelectColorFont(wxDC *dc, const StructWxColorFont &colorFont);
}


#endif // MDO_DRAWABLE_OBJECTS_HPP_DEVIS_MAKER
